package net.knarcraft.knarlib.util;

import org.junit.jupiter.api.Test;

import java.util.List;

import static org.junit.jupiter.api.Assertions.assertEquals;

/**
 * Tests for the tab-completion helper
 */
public class TabCompletionHelperTest {

    @Test
    public void getStringListUnfilteredTest() {
        List<String> input = List.of("a", "b", "c", "d");
        List<String> result = TabCompletionHelper.getStringList(input, "a,b,", null);
        List<String> expected = List.of("a,b,c", "a,b,d");
        assertEquals(expected, result);
    }

    @Test
    public void getStringListUnfilteredTest2() {
        List<String> input = List.of("a", "b", "c", "d");
        List<String> result = TabCompletionHelper.getStringList(input, "a,b", null);
        List<String> expected = List.of("a,b", "a,c", "a,d");
        assertEquals(expected, result);
    }

    @Test
    public void getStringListContainsTest() {
        List<String> input = List.of("ape", "ball", "car", "donut");
        List<String> result = TabCompletionHelper.getStringList(input, "ball,donut,r",
                TabCompletionHelper::filterMatchingContains);
        List<String> expected = List.of("ball,donut,car");
        assertEquals(expected, result);
    }

    @Test
    public void getStringListContainsTest2() {
        List<String> input = List.of("ape", "ball", "car", "donut");
        List<String> result = TabCompletionHelper.getStringList(input, "ball,donut,",
                TabCompletionHelper::filterMatchingContains);
        List<String> expected = List.of("ball,donut,ape", "ball,donut,car");
        assertEquals(expected, result);
    }

    @Test
    public void getStringListStartsWithTest() {
        List<String> input = List.of("ape", "ball", "car", "donut");
        List<String> result = TabCompletionHelper.getStringList(input, "ball,donut,a",
                TabCompletionHelper::filterMatchingStartsWith);
        List<String> expected = List.of("ball,donut,ape");
        assertEquals(expected, result);
    }

    @Test
    public void getStringListStartsWithTest2() {
        List<String> input = List.of("ape", "ball", "car", "donut");
        List<String> result = TabCompletionHelper.getStringList(input, "ball,donut,",
                TabCompletionHelper::filterMatchingStartsWith);
        List<String> expected = List.of("ball,donut,ape", "ball,donut,car");
        assertEquals(expected, result);
    }

    @Test
    public void filterMatchingContainsTest() {
        List<String> input = List.of("fish", "ape", "banana", "toy", "human", "stick", "potato", "detergent", "magazine");
        List<String> result = TabCompletionHelper.filterMatchingContains(input, "a");
        List<String> expected = List.of("ape", "banana", "human", "potato", "magazine");
        assertEquals(expected, result);
    }

    @Test
    public void filterMatchingStartsWithTest() {
        List<String> input = List.of("fish", "ape", "banana", "toy", "human", "stick", "potato", "detergent", "magazine");
        List<String> result = TabCompletionHelper.filterMatchingStartsWith(input, "a");
        List<String> expected = List.of("ape");
        assertEquals(expected, result);
    }

}
